#
#++
#
#  Copyright Digital Equipment Corporation 1996, 1997. All Rights Reserved.
#
#  Restricted Rights: Use, duplication, or disclosure by the U.S. Government
#  is subject to restrictions as set forth in subparagraph (c)(1)(ii) of DFARS
#  252.227-7013, or in FAR 52.227-19, or in FAR 52.227-14 Alt. III, as
#  applicable.
#
#  This software is proprietary to and embodies the confidential technology of
#  Digital Equipment Corporation. Possession, use, or copying of this software
#  and media is authorized only pursuant to a valid written license from
#  Digital or an authorized sublicensor.
#
#--
#
#++
#
#  MODULE:
#
#      D_RSVP.DDL
#
#  ABSTRACT:
#
#	DDL program to decode RSVP packets.  Packet formats are taken from
#
#		draft-ietf-rsvp-spec-16.txt,
#		draft-ietf-intserv-rsvp-use-02.txt
#
#	RSVP packets are carried in their own IP protocol, however older implementations
#	used to encapsulate inside UDP, this module supports both forms.
#
#	TBD:
#	o	Complete general Parameters
#	o	Service Specific Parameters
#
#
#  AUTHORS:
#
#      A A Hart
#
#  DATE:
#
#      13 Aug 1997
#
#  MODIFICATION HISTORY:
#
#
#
INCLUDE ddlinc.ddl


#
# Declarations
#
DECLARE PST pst_udps, pst_udpd, pst_ip;
DECLARE PST pst_rsvp_object, pst_rsvp_parameters, pst_rsvp_services;
DECLARE DECODE d_rsvp_object_unknown;
DECLARE NAME_TABLE IpProtocols, UdpPorts;

#
# RSVP Packet Offsets
#
CONSTANT
	UDP_sizeof	= 8,
	RSVP_verflgs	= 0,
	RSVP_type	= 1,
	RSVP_checksum	= 2,
	RSVP_send_ttl	= 4,
	RSVP_length	= 6,
	RSVP_sizeof 	= 8;

CONSTANT
	RSVP_msg_path	   = 1,
	RSVP_msg_resv	   = 2,
	RSVP_msg_patherr   = 3,
	RSVP_msg_resverr   = 4,
	RSVP_msg_pathtear  = 5,
	RSVP_msg_resvtear  = 6,
	RSVP_msg_resvconf  = 7;

# Object Classes
#
CONSTANT
	RSVP_class_session	= 1,
	RSVP_class_rsvp_hop	= 3,
	RSVP_class_time_values	= 5,
	RSVP_class_style	= 8,
	RSVP_class_flowspec	= 9,
	RSVP_class_filter_spec 	= 10,
	RSVP_class_sender_template = 11,
	RSVP_class_sender_tspec = 12,
	RSVP_class_adspec = 13;

CONSTANT
	RSVP_type_session_ipv4		= 1,
	RSVP_type_rsvp_hop_ipv4		= 1,
	RSVP_type_time_values		= 1,
	RSVP_type_filter_spec_ipv4 	= 1,
	RSVP_type_sender_template_ipv4	= 1,
	RSVP_type_sender_tspec_2	= 2,
	RSVP_type_adspec_2		= 2,
	RSVP_type_flowspec_2		= 2,
	RSVP_type_style			= 1;


# Object offsets
#
CONSTANT
	RSVP_obj_length	= 0,
	RSVP_obj_class	= 2,
	RSVP_obj_type	= 3,
	RSVP_obj_data	= 4,
	RSVP_obj_sizeof	= 4;

# SESSION Object Offsets
#
CONSTANT
	RSVP_obj_session_4dst		= RSVP_obj_data + 0,
	RSVP_obj_session_4protocol	= RSVP_obj_data + 4,
	RSVP_obj_session_4flags		= RSVP_obj_data + 5,
	RSVP_obj_session_4port		= RSVP_obj_data + 6;

# RSVP_HOP Object Offsets
#
CONSTANT
	RSVP_obj_hop_addr		= RSVP_obj_data + 0,
	RSVP_obj_hop_handle		= RSVP_obj_data + 4;

# TIME_VALUES Object Offsets
#
CONSTANT
	RSVP_obj_time_values_refresh	= RSVP_obj_data + 0;

# SENDER_TSPEC Object Offsets
#
CONSTANT
	RSVP_obj_sender_tspec_version		= RSVP_obj_data + 0,
	RSVP_obj_sender_tspec_length		= RSVP_obj_data + 2,
	RSVP_obj_sender_tspec_sizeof		= RSVP_obj_data + 4;

# ADSPEC Object Offsets
#
CONSTANT
	RSVP_obj_adspec_version	= RSVP_obj_data + 0,
	RSVP_obj_adspec_length	= RSVP_obj_data + 2,
	RSVP_obj_adspec_sizeof	= RSVP_obj_data + 4;

# FLOWSPEC Object Offsets
#
CONSTANT
	RSVP_obj_flowspec_version	= RSVP_obj_data + 0,
	RSVP_obj_flowspec_length	= RSVP_obj_data + 2,
	RSVP_obj_flowspec_sizeof	= RSVP_obj_data + 4;

# FILTER_SPEC Object Offsets
#
CONSTANT
	RSVP_obj_filter_spec_4src	= RSVP_obj_data + 0,
	RSVP_obj_filter_spec_4port	= RSVP_obj_data + 6;

CONSTANT
	RSVP_protocol	= 46,
	RSVP_udp_port1	= 1698,
	RSVP_udp_port2	= 1699;


# Parameter Offsets.
#
CONSTANT
	RSVP_param_id		= 0,
	RSVP_param_flags	= 1,
	RSVP_param_length	= 2,
	RSVP_param_data		= 4,
	RSVP_param_sizeof	= 4;


# Parameter ID's
#
CONSTANT
	RSVP_param_null		= 0,
	RSVP_param_nonishop	= 2,
	RSVP_param_nishop	= 4,
	RSVP_param_apbw		= 6,
	RSVP_param_mpl		= 8,
	RSVP_param_pmtu		= 10,
	RSVP_param_tb		= 127;

CONSTANT
	RSVP_param_null_sizeof		= 4 + 0,
	RSVP_param_nonishop_sizeof	= 4 + 0,
	RSVP_param_nishop_sizeof	= 4 + 4,
	RSVP_param_apbw_sizeof		= 4 + 4,
	RSVP_param_mpl_sizeof		= 4 + 4,
	RSVP_param_pmtu_sizeof		= 4 + 4,
	RSVP_param_tb_sizeof		= 4 + (4 * 5);


# Service Offsets.
#
CONSTANT
	RSVP_service_number	= 0,
	RSVP_service_break	= 1,
	RSVP_service_length	= 2,
	RSVP_service_sizeof	= 4;


# GOTO labels
#
CONSTANT
	RSVP_goto_object	= 1;


# Packet Type Names
#
NAME_LIST rsvpMessageType {
	TITLE = "Message Type",
	OUT_OF_RANGE = "unknown"
	} =

		"Illegal",	# 0
		"Path",		# 1
		"Resv",		# 2
		"PathErr",	# 3
		"ResvErr",	# 4
		"PathTear",	# 5
		"ResvTear",	# 6
		"ResvConf";	# 7


# Object Classes
# Note that we assume for the moment that the class numbers are
# contigious, if new non-contigious classes are added in the future
# this may need to become a NAME_TABLE.
#
NAME_LIST rsvpObjectClass {
	TITLE = "Object Class",
	OUT_OF_RANGE = "UNKNOWN"
	} =

		"NULL",		# 0
		"SESSION",	# 1
		"UNASSIGNED",	# 2
		"RSVP_HOP",	# 3
		"INTEGRITY",	# 4
		"TIME_VALUES",	# 5
		"ERROR_SPEC",	# 6
		"SCOPE",	# 7
		"STYLE",	# 8
		"FLOWSPEC",	# 9
		"FILTER_SPEC",	# 10
		"SENDER_TEMPLATE",	# 11
		"SENDER_TSPEC",	# 12
		"ADSPEC",	# 13
		"POLICY_DATA",	# 14
		"RESV_CONFIRM"	# 15
		;


# Service Names
#
NAME_LIST rsvpServiceName {
	TITLE = "  Service Number",
	OUT_OF_RANGE = "(Unknown)"
	} =
		"NULL",					# 0
		"1 (Default General Parameters)",	# 1
		"2 (Guaranteed Service)",		# 2
		"3 (Unknown)",				# 3
		"4 (Unknown)",				# 4
		"5 (Controlled Load)"			# 5
		;


# Style Sharing Control
#
NAME_LIST rsvpStyleSharingControl {
	TITLE = "  Sharing Control"
	} =

		"Reserved",			# 0x0
		"Distinct reservations",	# 0x1
		"Shared reservations",		# 0x2
		"Reserved"			# 0x3
		;


# Style Sender Control
#
NAME_LIST rsvpStyleSenderControl {
	TITLE = "  Sender Selection Control"
	} =

		"Reserved",			# 0x0
		"Wildcard",			# 0x1
		"Explicit"			# 0x2
		;


# Session Object Flags
#
FLAG_TABLE rsvpSessionFlags {
	TITLE = "  Flags"
	} =
		0x01 = "E_Police"
		;

# GOTO PST
# Switch on the value in $31 to jump to the correct PST.
#
PST pst_rsvp_goto( $31 ) =

		RSVP_goto_object -> pst_rsvp_object
		;





# ++++++++++++++++++++++++++++++++++ Parameter Blocks ++++++++++++++++++++++++++++++++++++
# Decodes the general paramter blocks in the 1-127 range (the 128-154 range parameters are
# particular to a service type).   The parameter header is shown below, the data immediately
# follows the header.
#
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |      id       |   flags       |          length               |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#
# The pst_rsvp_parameters PST is the entry point for this section, the $20 variable is an
# input and contains the size of the section in bytes (the section contains a contigious
# list of parameters). 
#
# Each decode stores the actual length of the parameter (taken from the packet) in the $1
# variable.  This value is subtracted from the $20 variable (to decrement the remaining
# number of bytes in this section).  The decode also computes the difference between the
# expected length of the parameter and irs actual length and dumps any residual bytes if
# any (this should catch the case where the parameter data is expanded in the future).
#

# Decode Unknwon Parameter Block
# This is the default parameter block decoder, just print out the numbers and dump the data
# as hex bytes.
#
DECODE d_rsvp_param_unknown(0) {
	NEXT = pst_rsvp_parameters,
	CONSUME = $1
	} =

		$1 = ( word( RSVP_param_length ) + 1 ) * 4,
		$20 = $20 - $1,

		print( "    Parameter ID,Length,Flags", "%d,%d,0x%X",
			byte( RSVP_param_id ),
			word( RSVP_param_length ),
			byte( RSVP_param_flags )),

		dump_hex_bytes( RSVP_param_sizeof, $1 - 4, "                    " )
		;


# Decode NULL Parameter
#
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |    0 (e)      |    (f)        |           0 (g)               |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#
DECODE d_rsvp_param_null( RSVP_param_null ) {
	NEXT = pst_rsvp_parameters,
	CONSUME = $1	
	} =
		
		$1 = ( word( RSVP_param_length ) + 1 ) * 4,
		$2 = $1 - RSVP_param_null_sizeof,
		$20 = $20 - $1,

		print( "    NULL (length,flags)", "%d,0x%X", word(RSVP_param_length), byte(RSVP_param_flags)),
		dump_hex_bytes( RSVP_param_null_sizeof, $2, "                  " ) {CONDITION = gt($2)}
		;


# Decode NON-IS Hop Parameter
#
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |    2 (e)      |    (f)        |           0 (g)               |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#
DECODE d_rsvp_param_nonishop( RSVP_param_nonishop ) {
	NEXT = pst_rsvp_parameters,
	CONSUME = $1	
	} =
		
		$1 = ( word( RSVP_param_length ) + 1 ) * 4,
		$2 = $1 - RSVP_param_nonishop_sizeof,
		$20 = $20 - $1,

		boolean( byte( RSVP_param_flags ), 0, "    NON-IS_HOP" ),
		print( "      Length, Flags", "%d, 0x%X", word( RSVP_param_length ), byte( RSVP_param_flags )),
		dump_hex_bytes( RSVP_param_nonishop_sizeof, $2, "                  " ) {CONDITION = gt($2)}
		;


# Decode IS Hop Count Parameter
#
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |    4 (e)      |    (f)        |           1 (g)               |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |        IS hop cnt (32-bit unsigned integer)                   |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#
DECODE d_rsvp_param_nishop( RSVP_param_nishop ) {
	NEXT = pst_rsvp_parameters,
	CONSUME = $1	
	} =
		
		$1 = ( word( RSVP_param_length ) + 1 ) * 4,
		$2 = $1 - RSVP_param_nishop_sizeof,
		$20 = $20 - $1,

		decimal( long( RSVP_param_sizeof ), 0, "    NUMBER_OF_IS_HOPS" ),
		print( "      Length, Flags", "%d, 0x%X", word( RSVP_param_length ), byte( RSVP_param_flags )),
		dump_hex_bytes( RSVP_param_nishop_sizeof, $2, "                  " ) {CONDITION = gt($2)}
		;


# Decode Available Path Bandwidth Parameter
#
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |    4 (e)      |    (f)        |           1 (g)               |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |        Path b/w estimate  (32-bit IEEE floating point number) |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#
DECODE d_rsvp_param_apbw( RSVP_param_apbw ) {
	NEXT = pst_rsvp_parameters,
	CONSUME = $1	
	} =
		
		$1 = ( word( RSVP_param_length ) + 1 ) * 4,
		$2 = $1 - RSVP_param_apbw_sizeof,
		$20 = $20 - $1,

		float( long( RSVP_param_sizeof ), 0, "    AVAILABLE_PATH_BANDWIDTH" ),
		print( "      Length, Flags", "%d, 0x%X", word( RSVP_param_length ), byte( RSVP_param_flags )),
		dump_hex_bytes( RSVP_param_apbw_sizeof, $2, "                  " ) {CONDITION = gt($2)}
		;


# Decode Minimum Path Latency Parameter
#
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |     8 (k)     |    (l)        |           1 (m)               |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |        Minimum path latency (32-bit integer)                  |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#
DECODE d_rsvp_param_mpl( RSVP_param_mpl ) {
	NEXT = pst_rsvp_parameters,
	CONSUME = $1	
	} =
		
		$1 = ( word( RSVP_param_length ) + 1 ) * 4,
		$2 = $1 - RSVP_param_mpl_sizeof,
		$20 = $20 - $1,

		decimal( long( RSVP_param_sizeof ), 0, "    MINIMUM_PATH_LATENCY" ),
		print( "      Length, Flags", "%d, 0x%X", word( RSVP_param_length ), byte( RSVP_param_flags )),
		dump_hex_bytes( RSVP_param_mpl_sizeof, $2, "                  " ) {CONDITION = gt($2)}
		;


# Decode Path MTU Parameter
#
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |     10 (n)    |      (o)      |           1 (p)               |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |      Composed MTU (32-bit unsigned integer)                   |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#
DECODE d_rsvp_param_pmtu( RSVP_param_pmtu ) {
	NEXT = pst_rsvp_parameters,
	CONSUME = $1	
	} =
		
		$1 = ( word( RSVP_param_length ) + 1 ) * 4,
		$2 = $1 - RSVP_param_pmtu_sizeof,
		$20 = $20 - $1,

		decimal( long( RSVP_param_sizeof ), 0, "    PATH_MTU" ),
		print( "      Length, Flags", "%d, 0x%X", word( RSVP_param_length ), byte( RSVP_param_flags )),
		dump_hex_bytes( RSVP_param_pmtu_sizeof, $2, "                  " ) {CONDITION = gt($2)}
		;



# Decode the Token Bucket TSPEC
#
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |   127 (e)     |    0 (f)      |             5 (g)             |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |  Token Bucket Rate [r] (32-bit IEEE floating point number)    |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |  Token Bucket Size [b] (32-bit IEEE floating point number)    |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |  Peak Data Rate [p] (32-bit IEEE floating point number)       |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |  Minimum Policed Unit [m] (32-bit integer)                    |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |  Maximum Packet Size [M]  (32-bit integer)                    |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#
#
#     (a) - Message format version number (0)
#     (b) - Overall length (7 words not including header)
#     (c) - Service header, service number 1 (default/global information)
#     (d) - Length of service 1 data, 6 words not including header
#     (e) - Parameter ID, parameter 127 (Token_Bucket_TSpec)
#     (f) - Parameter 127 flags (none set)
#     (g) - Parameter 127 length, 5 words not including header
#
DECODE d_rsvp_param_tb( RSVP_param_tb ) {
	NEXT = pst_rsvp_parameters,
	CONSUME = $1	
	} =

		$1 = ( word( RSVP_param_length ) + 1 ) * 4,
		$2 = $1 - RSVP_param_tb_sizeof,
		$20 = $20 - $1,

		print( "    TOKEN_BUCKET_TSPEC", "", 0 ),
		print( "      Length, Flags", "%d, 0x%X", word( RSVP_param_length ), byte( RSVP_param_flags )),
		float( long( RSVP_param_data + 0 ), 0, "      [r] Rate" ),
		float( long( RSVP_param_data + 4 ), 0, "      [b] Size" ),
		float( long( RSVP_param_data + 8 ), 0, "      [p] Peak rate" ),
		decimal( long( RSVP_param_data + 12 ), 0, "      [m] Min Policed Unit" ),
		decimal( long( RSVP_param_data + 16 ), 0, "      [M] Max Packet Size" ),
		dump_hex_bytes( RSVP_param_pmtu_sizeof, $2, "                  " ) {CONDITION = gt($2)}
		;



# Switch on the Parameter ID (for Service Number 5)
# Call the default parameter decode for any unrecognised parameter type.
# Input:
#	$29 = Service Number
#	$30 = Remaining bytes in this services section
#
PST pst_rsvp_parameter_id_5( byte( RSVP_param_id ) ) {
	DEFAULT = d_rsvp_param_unknown
	} =
		d_rsvp_param_null,
		d_rsvp_param_nonishop,
		d_rsvp_param_nishop,
		d_rsvp_param_apbw,
		d_rsvp_param_mpl,
		d_rsvp_param_pmtu,
		d_rsvp_param_tb
		;

# Switch on the Parameter ID (for Service Number 2)
# Call the default parameter decode for any unrecognised parameter type.
# Input:
#	$29 = Service Number
#	$30 = Remaining bytes in this services section
#
PST pst_rsvp_parameter_id_2( byte( RSVP_param_id ) ) {
	DEFAULT = d_rsvp_param_unknown
	} =
		d_rsvp_param_null,
		d_rsvp_param_nonishop,
		d_rsvp_param_nishop,
		d_rsvp_param_apbw,
		d_rsvp_param_mpl,
		d_rsvp_param_pmtu,
		d_rsvp_param_tb
		;

# Switch on the Parameter ID (for Service Number 1)
# Call the default parameter decode for any unrecognised parameter type.
# Input:
#	$29 = Service Number
#	$30 = Remaining bytes in this services section
#
PST pst_rsvp_parameter_id_1( byte( RSVP_param_id ) ) {
	DEFAULT = d_rsvp_param_unknown
	} =
		d_rsvp_param_null,
		d_rsvp_param_nonishop,
		d_rsvp_param_nishop,
		d_rsvp_param_apbw,
		d_rsvp_param_mpl,
		d_rsvp_param_pmtu,
		d_rsvp_param_tb
		;


# Switch on the service number in $29 to invoke the appropriate parameter process
# PST (there is a parameter pst for each service number so that the service specific
# parameters can be decoded appropriately).
# Input:
#	$29 = Service Number
#	$30 = Remaining bytes in this services section
#
PST pst_rsvp_parameter_id( $29 ) =
		0x1 -> pst_rsvp_parameter_id_1,
		0x2 -> pst_rsvp_parameter_id_2,
		0x5 -> pst_rsvp_parameter_id_5
		;


# Switch on the $20 variable until there is no more parameters left in this section
# Input:
#	$29 = Service Number
#	$30 = Remaining bytes in this services section
#
PST pst_rsvp_parameters( gt($20) ) =
		0x0 -> pst_rsvp_services,
		0x1 -> pst_rsvp_parameter_id
		;

# ---------------------------------- Parameter Blocks ------------------------------------


# ++++++++++++++++++++++++++++++++++ Service Blocks ++++++++++++++++++++++++++++++++++++
# Decode each service block in a group, the size of the group is stored in the $30 variable.
#
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |     number    |b| reserved    |             length            |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#
# The pst_rsvp_services PST is the entry point for this section, the $30 variable is an
# input and contains the size of the section in bytes (the section contains a contigious
# list of service blocks). 

#
# Decode Unknwon Service
# This is the default service decoder, just print out the numbers and then invoke the
# parameter list decoder.
# Input:
#	$30 = Remaining bytes in this services section
# Output:
#	$20 = Size (in bytes) of this service block (excluding header)
#	$29 = Service number
#	$30 = $30 - $20
#
DECODE d_rsvp_service_unknown(0) {
	NEXT = pst_rsvp_parameters,
	CONSUME = RSVP_service_sizeof
	} =

		$20 = word( RSVP_service_length ) * 4,
		$30 = $30 - $20,
		$29 = byte( RSVP_service_number ),

		nl(0),
		list( byte( RSVP_service_number ), 0, rsvpServiceName ),
		decimal( bitfield_byte( 1, 7, RSVP_service_break ), 0, "  Break Flag" ),
		decimal( word( RSVP_service_length ), 0, "  Service Length" ),
		;


# Switch on the Service number
# Call the default service decode for any unrecognised service.
#
PST pst_rsvp_service_number( byte( RSVP_service_number ) ) {
	DEFAULT = d_rsvp_service_unknown
	} =
		;


# Switch on the $30 variable until there is no more parameters left in this section.
# When done jump to the pst_rsvp_goto pst.
#
PST pst_rsvp_services( gt($30) ) =

		0x0 -> pst_rsvp_goto,
		0x1 -> pst_rsvp_service_number
		;

# ---------------------------------- Service Blocks ------------------------------------


# ++++++++++++++++++++++++++++++++++ FLOWSPEC Object ++++++++++++++++++++++++++++++++++++
#3.2.2. FLOWSPEC Object when Requesting Guaranteed Service
#
#   The format of an RSVP FLOWSPEC object originating at a receiver
#   requesting Guaranteed service is shown below. The flowspec object
#   used to request guaranteed service carries a TSpec and RSpec
#   specifying the traffic parameters of the flow desired by the
#   receiver.
#
#   Each of the TSpec and RSpec fields is represented using the preferred
#   concrete representation specified in the 'Invocation Information'
#   section of [RFCG]. The value of 2 for the service header identifier
#   (field (c) in the picture below) indicates that Guaranteed service is
#   being requested.
#
#        31           24 23           16 15            8 7             0
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       | 0 (a) |    Unused             |            10 (b)             |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |                                                               |
#
DECODE d_rsvp_object_flowspec_2( RSVP_type_flowspec_2 ) {
	CONSUME = RSVP_obj_sizeof + 4,
	NEXT = pst_rsvp_services,
	PLEVEL = HL_RSVP
	} =

		$31 = RSVP_goto_object,
		$30 = word( RSVP_obj_length ) - (RSVP_obj_flowspec_sizeof + RSVP_service_sizeof),
		
		nl(0),
		print( "Type Class and (Length)", "FLOWSPEC (%d)", $30 ),
		decimal( byte( RSVP_obj_flowspec_version ), 0, "  Format Version" ),
		decimal( word( RSVP_obj_flowspec_length ), 0, "  Overall Length" )
		;


# PST: pst_rsvp_object_flowspec
# Switch on the class type and invoke the appropriate decode, if the type is unrecognised
# then call d_rsvp_object_unknown
#
PST pst_rsvp_object_flowspec( byte( RSVP_obj_type ) ) {
	DEFAULT = d_rsvp_object_unknown
	} =
		d_rsvp_object_flowspec_2
		;


# -------------------------------- FLOWSPEC Object -------------------------------------





# ++++++++++++++++++++++++++++++++++ ADSPEC Object ++++++++++++++++++++++++++++++++++++
#3.3.1. RSVP ADSPEC format
#
#   The basic ADSPEC format is shown below. The message header and the
#   default general parameters fragment are always present. The fragments
#   for Guaranteed or Controlled-Load service may be omitted if the
#   service is not to be used by the RSVP session. Additional data
#   fragments will be added if new services are defined.
#
#       31           24 23            16 15            8 7             0
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       | 0 (a) |      reserved         |  Msg length - 1 (b)           |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |                                                               |
#       |    Default General Parameters fragment (Service 1)  (c)       |
#       |    (Always Present)                                           |
#       |                                                               |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |                                                               |
#       |    Guaranteed Service Fragment (Service 2)    (d)             |
#       |    (Present if application might use Guaranteed Service)      |
#       |                                                               |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#       |                                                               |
#       |    Controlled-Load Service Fragment (Service 5)  (e)          |
#       |    (Present if application might use Controlled-Load Service) |
#       |                                                               |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#
#     (a) - Message format version number (0)
#     (b) - Overall message length not including header word
#     (c, d, e) - Data fragments
#
#
DECODE d_rsvp_object_adspec_2( RSVP_type_adspec_2 ) {
	CONSUME = RSVP_obj_sizeof + 4,
	NEXT = pst_rsvp_services,
	PLEVEL = HL_RSVP
	} =

		$31 = RSVP_goto_object,
		$30 = word( RSVP_obj_length ) - (RSVP_obj_adspec_sizeof + RSVP_service_sizeof),
		
		nl(0),
		print( "Type Class and (Length)", "ADSPEC (%d)", $30 ),
		decimal( byte( RSVP_obj_adspec_version ), 0, "  Format Version" ),
		decimal( word( RSVP_obj_adspec_length ), 0, "  Overall Length" )
		;


# PST: pst_rsvp_object_adspec
# Switch on the class type and invoke the appropriate decode, if the type is unrecognised
# then call d_rsvp_object_unknown
#
PST pst_rsvp_object_adspec( byte( RSVP_obj_type ) ) {
	DEFAULT = d_rsvp_object_unknown
	} =
		d_rsvp_object_adspec_2
		;


# -------------------------------- ADSPEC Object -------------------------------------


# ++++++++++++++++++++++++++++++++++ SENDER_TSPEC Object ++++++++++++++++++++++++++++++++++++
# DECODE: d_rsvp_object_sender_tspec_2
# Handle the SENDER_TSPEC class.
#
#3.1. RSVP SENDER_TSPEC Object
#
#   The RSVP SENDER_TSPEC object carries information about a data
#   source's generated traffic. The required RSVP SENDER_TSPEC object
#   contains a global Token_Bucket_TSpec parameter (service_number 1,
#   parameter 127, as defined in [RFCGP]). This TSpec carries traffic
#   information usable by either the Guaranteed or Controlled-Load QoS
#   control services.
#
#        31           24 23           16 15            8 7             0
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   1   | 0 (a) |    reserved           |             7 (b)             |
#       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   2   |
#
#
DECODE d_rsvp_object_sender_tspec_2( RSVP_type_sender_tspec_2 ) {
	CONSUME = RSVP_obj_sizeof + 4,
	NEXT = pst_rsvp_services,
	PLEVEL = HL_RSVP
	} =
		

		$31 = RSVP_goto_object,
		$30 = word( RSVP_obj_length ) - (RSVP_obj_sender_tspec_sizeof + RSVP_service_sizeof),
		
		nl(0),
		print( "Type Class and (Length)", "SENDER_TSPEC (%d)", word( RSVP_obj_length ) ),
		decimal( byte( RSVP_obj_sender_tspec_version ), 0, "  Format Version" ),
		decimal( word( RSVP_obj_sender_tspec_length ), 0, "  Overall Length" )
		;


# PST: pst_rsvp_object_sender_tspec
# Switch on the class type and invoke the appropriate decode, if the type is unrecognised
# then call d_rsvp_object_unknown
#
PST pst_rsvp_object_sender_tspec( byte( RSVP_obj_type ) ) {
	DEFAULT = d_rsvp_object_unknown
	} =
		d_rsvp_object_sender_tspec_2
		;

# -------------------------------- SENDER_TEMPLATE Object -------------------------------------


# ++++++++++++++++++++++++++++++++++ SENDER_TEMPLATE Object ++++++++++++++++++++++++++++++++++++
# DECODE: d_rsvp_object_sender_template_ipv4
# Handle the SENDER_TEMPLATE class.
#
DECODE d_rsvp_object_sender_template_ipv4( RSVP_type_sender_template_ipv4 ) {
	CONSUME = word( RSVP_obj_length ),
	NEXT = pst_rsvp_object,
	PLEVEL = HL_RSVP
	} =
		
		nl(0),
		print( "Type Class and (Length)", "IPv4 SENDER_TEMPLATE (%d)", word( RSVP_obj_length ) ),
		ip_address( long( RSVP_obj_filter_spec_4src ), TRANSLATE, "  Source Address" ),
		name( "  Source Port", UdpPorts, word( RSVP_obj_filter_spec_4port ) )
		;


# PST: pst_rsvp_object_sender_template
# Switch on the class type and invoke the appropriate decode, if the type is unrecognised
# then call d_rsvp_object_unknown
#
PST pst_rsvp_object_sender_template( byte( RSVP_obj_type ) ) {
	DEFAULT = d_rsvp_object_unknown
	} =
		d_rsvp_object_sender_template_ipv4
		;

# -------------------------------- SENDER_TEMPLATE Object -------------------------------------


# ++++++++++++++++++++++++++++++++++ FILTER_SPEC Object ++++++++++++++++++++++++++++++++++++
# DECODE: d_rsvp_object_filter_spec_ipv4
# Handle the FILTER_SPEC class.
#      o    IPv4 FILTER_SPEC object: Class = 10, C-Type = 1
#
#           +-------------+-------------+-------------+-------------+
#           |               IPv4 SrcAddress (4 bytes)               |
#           +-------------+-------------+-------------+-------------+
#           |    //////   |    //////   |          SrcPort          |
#           +-------------+-------------+-------------+-------------+
#
DECODE d_rsvp_object_filter_spec_ipv4( RSVP_type_filter_spec_ipv4 ) {
	CONSUME = word( RSVP_obj_length ),
	NEXT = pst_rsvp_object,
	PLEVEL = HL_RSVP
	} =
		
		nl(0),
		print( "Type Class and (Length)", "IPv4 FILTER_SPEC (%d)", word( RSVP_obj_length ) ),
		
		ip_address( long( RSVP_obj_filter_spec_4src ), TRANSLATE, "  Source Address" ),
		name( "  Source Port", UdpPorts, word( RSVP_obj_filter_spec_4port ) )
		;


# PST: pst_rsvp_object_filter_spec
# Switch on the class type and invoke the appropriate decode, if the type is unrecognised
# then call d_rsvp_object_unknown
#
PST pst_rsvp_object_filter_spec( byte( RSVP_obj_type ) ) {
	DEFAULT = d_rsvp_object_unknown
	} =
		d_rsvp_object_filter_spec_ipv4
		;

# -------------------------------- FILTER_SPEC Object -------------------------------------

# ++++++++++++++++++++++++++++++++++ TIME_VALUES Object ++++++++++++++++++++++++++++++++++++
# DECODE: d_rsvp_object_time_values_refresh
# Handle the TIME_VALUES class.
#      o    TIME_VALUES Object: Class = 5, C-Type = 1
#
#
#           +-------------+-------------+-------------+-------------+
#           |                   Refresh Period R                    |
#           +-------------+-------------+-------------+-------------+
#
DECODE d_rsvp_object_time_values_refresh( RSVP_type_time_values ) {
	CONSUME = word( RSVP_obj_length ),
	NEXT = pst_rsvp_object,
	PLEVEL = HL_RSVP
	} =
		
		nl(0),
		print( "Type Class and (Length)", "TIME_VALUES (%d)", word( RSVP_obj_length ) ),
		time( long( RSVP_obj_time_values_refresh ), MILLISECONDS, "  Refresh Period" )
		;


# PST: pst_rsvp_object_time_values
# Switch on the class type and invoke the appropriate decode, if the type is unrecognised
# then call d_rsvp_object_unknown
#
PST pst_rsvp_object_time_values( byte( RSVP_obj_type ) ) {
	DEFAULT = d_rsvp_object_unknown
	} =
		d_rsvp_object_time_values_refresh
		;

# -------------------------------- TIME_VALUES Object -------------------------------------



# ++++++++++++++++++++++++++++++++++ RSVP_HOP Object ++++++++++++++++++++++++++++++++++++
# DECODE: d_rsvp_object_rsvp_hop_ipv4
# Handle the RSVP_HOP class.
#
#      o    IPv4 RSVP_HOP object: Class = 3, C-Type = 1
#
#           +-------------+-------------+-------------+-------------+
#           |             IPv4 Next/Previous Hop Address            |
#           +-------------+-------------+-------------+-------------+
#           |                 Logical Interface Handle              |
#           +-------------+-------------+-------------+-------------+
#
DECODE d_rsvp_object_rsvp_hop_ipv4( RSVP_type_rsvp_hop_ipv4 ) {
	CONSUME = word( RSVP_obj_length ),
	NEXT = pst_rsvp_object,
	PLEVEL = HL_RSVP
	} =
		
		nl(0),
		print( "Type Class and (Length)", "IPv4 RSVP_HOP (%d)", word( RSVP_obj_length ) ),
		
		ip_address( long( RSVP_obj_hop_addr ), TRANSLATE, "  Next/Previous Hop" ),
		hex( long( RSVP_obj_hop_handle ), 0, "  Logical Interface Handle" )
		;


# PST: pst_rsvp_object_rsvp_hop
# Switch on the class type and invoke the appropriate decode, if the type is unrecognised
# then call d_rsvp_object_unknown
#
PST pst_rsvp_object_rsvp_hop( byte( RSVP_obj_type ) ) {
	DEFAULT = d_rsvp_object_unknown
	} =
		d_rsvp_object_rsvp_hop_ipv4
		;

# -------------------------------- RSVP_HOP Object -------------------------------------



# ++++++++++++++++++++++++++++++++++ SESSION Object ++++++++++++++++++++++++++++++++++++
# DECODE: d_rsvp_object_session_ipv4
# Handle the SESSION IPv4/UDP object.
#
#      o    IPv4/UDP SESSION object: Class = 1, C-Type = 1
#
#           +-------------+-------------+-------------+-------------+
#           |             IPv4 DestAddress (4 bytes)                |
#           +-------------+-------------+-------------+-------------+
#           | Protocol Id |    Flags    |          DstPort          |
#           +-------------+-------------+-------------+-------------+
#
DECODE d_rsvp_object_session_ipv4( RSVP_type_session_ipv4 ) {
	CONSUME = word( RSVP_obj_length ),
	NEXT = pst_rsvp_object,
	PLEVEL = HL_RSVP
	} =
		
		nl(0),
		print( "Type Class and (Length)", "IPv4/UDP SESSION (%d)", word( RSVP_obj_length ) ),
		
		ip_address( long( RSVP_obj_session_4dst ), TRANSLATE, "  Destination Address" ),
		name( "  Protocol", IpProtocols, byte( RSVP_obj_session_4protocol ) ),
		name( "  Destination Port", UdpPorts, word( RSVP_obj_session_4port ) ),
		flags( byte( RSVP_obj_session_4flags ), 0, rsvpSessionFlags )
		;


# PST: pst_rsvp_object_session
# Switch on the class type and invoke the appropriate decode, if the type is unrecognised
# then call d_rsvp_object_unknown
#
PST pst_rsvp_object_session( byte( RSVP_obj_type ) ) {
	DEFAULT = d_rsvp_object_unknown
	} =
		d_rsvp_object_session_ipv4
		;

# -------------------------------- SESSION Object -------------------------------------



# ++++++++++++++++++++++++++++++++++ STYLE Object ++++++++++++++++++++++++++++++++++++
# DECODE: d_rsvp_object_style
# Handle the STYLE object.
#
#      o    STYLE object: Class = 8, C-Type = 1
#
#           +-------------+-------------+-------------+-------------+
#           |   Flags     |              Option Vector              |
#           +-------------+-------------+-------------+-------------+
#
DECODE d_rsvp_object_style_1( RSVP_type_style ) {
	CONSUME = word( RSVP_obj_length ),
	NEXT = pst_rsvp_object,
	PLEVEL = HL_RSVP
	} =
		
		nl(0),
		print( "Type Class and (Length)", "STYLE (%d)", word( RSVP_obj_length ) ),
		
		hex( byte( RSVP_obj_data ), 0, "  Flags" ),
		hex(  bitfield_long( 19, 5, RSVP_obj_data ), 0, "  Reserved bits" ),
		list( bitfield_long( 2, 3, RSVP_obj_data ), 0, rsvpStyleSharingControl ),
		list( bitfield_long( 3, 0, RSVP_obj_data ), 0, rsvpStyleSenderControl ),
		;


# PST: pst_rsvp_object_style
# Switch on the class type and invoke the appropriate decode, if the type is unrecognised
# then call d_rsvp_object_unknown
#
PST pst_rsvp_object_style( byte( RSVP_obj_type ) ) {
	DEFAULT = d_rsvp_object_unknown
	} =
		d_rsvp_object_style_1
		;

# -------------------------------- STYLE Object -------------------------------------





# DECODE: d_rsvp_class_unknown
# Handle unknown classes.  Simply print out a hex dump of the data contents.
#
DECODE d_rsvp_object_unknown( 0 ) {
	CONSUME = word( RSVP_obj_length ),
	NEXT = pst_rsvp_object,
	PLEVEL = HL_RSVP
	} =
		$1 = word( RSVP_obj_length ),

		nl(0),
		list( byte( RSVP_obj_class ), 0, rsvpObjectClass ),
		decimal( byte( RSVP_obj_type ), 0, "Object Type" ),
		decimal( $1, 0, "Object Length" ),
		dump_hex_bytes( RSVP_obj_data, $1 - RSVP_obj_data, "             " )
		;



# PST: pst_rsvp_object
# Switch on the class number of the current object and invoke another PST which
# will switch on the class type.
# Input:
#	CO = object
#
PST pst_rsvp_object( byte( RSVP_obj_class ) ) {
	DEFAULT = d_rsvp_object_unknown
	} =
		RSVP_class_session -> pst_rsvp_object_session,
		RSVP_class_rsvp_hop -> pst_rsvp_object_rsvp_hop,
		RSVP_class_time_values -> pst_rsvp_object_time_values,
		RSVP_class_filter_spec -> pst_rsvp_object_filter_spec,
		RSVP_class_sender_template -> pst_rsvp_object_sender_template,
		RSVP_class_sender_tspec -> pst_rsvp_object_sender_tspec,
		RSVP_class_adspec -> pst_rsvp_object_adspec,
		RSVP_class_flowspec -> pst_rsvp_object_flowspec,
		RSVP_class_style -> pst_rsvp_object_style
		;



# DECODE: d_rsvp_resv
# Decode a resv message.  The message consists of a sequence of objects.
# Input:
#	Co = RSVP header
#	$10 = message length
#
DECODE d_rsvp_resv( RSVP_msg_resv ) {
	TITLE = "Resv",
	NEXT = pst_rsvp_object,
	CONSUME = RSVP_sizeof
	} =

		;


# DECODE: d_rsvp_pathtear
# Decode a pathtear message.  The message consists of a sequence of objects.
# Input:
#	Co = RSVP header
#	$10 = message length
#
DECODE d_rsvp_pathtear( RSVP_msg_pathtear ) {
	TITLE = "PathTear",
	NEXT = pst_rsvp_object,
	CONSUME = RSVP_sizeof
	} =

		;


# DECODE: d_rsvp_path
# Decode a path message.  The message consists of a sequence of objects.
# Input:
#	Co = RSVP header
#	$10 = message length
#
DECODE d_rsvp_path( RSVP_msg_path ) {
	TITLE = "Path",
	NEXT = pst_rsvp_object,
	CONSUME = RSVP_sizeof
	} =

		;



# PST: pst_rsvp_type
# Switch on the RSVP packet type.
# Input:
#	CO = RSVP header
#
PST pst_rsvp_type( byte( RSVP_type ) ) {
	TITLE = "RSVP Type"
	} =
		d_rsvp_path,
		d_rsvp_resv,
		d_rsvp_pathtear
		;


# DECODE d_rsvp
# Main RSVP packet decode.
# Input:
# 	CO = RSVP header
# Output:
#	$10 = rsvp message length
#
DECODE d_rsvp(0) {
	NEXT = pst_rsvp_type,
	PLEVEL = HL_RSVP
	} =
		$10 = word( RSVP_length ),

		decimal( bitfield_byte( 4, 4, RSVP_verflgs ), 0, "Version" ),
		hex( bitfield_byte( 4, 0, RSVP_verflgs ), 0, "Flags" ),
		list( byte( RSVP_type), 0, rsvpMessageType ),
		decimal( byte( RSVP_send_ttl ), 0, "Send TTL" ),
		decimal( $10, 0, "Length" ),
		ip_checksum( RSVP_checksum, ($10 << 16) + RSVP_checksum, "Verified Checksum" )
		;


# DECODE d_rsvp_n
# This decode attaches to the UDP port decode (both source and destination) for
# both ports (1698 and 1699) and passes control to the common DECODE.  The decode
# also removes the UDP header.
#
DECODE d_rsvp_1( RSVP_udp_port1 ) {
	PLEVEL = HL_RSVP,
	TITLE = "Encapsulated RSVP", BRIEF_TITLE = "EncapRSVP", ATTACH = pst_udps, CONSUME = UDP_sizeof, NEXT = d_rsvp } =;
DECODE d_rsvp_2( RSVP_udp_port2 ) {
	PLEVEL = HL_RSVP,
	TITLE = "Encapsulated RSVP", BRIEF_TITLE = "EncapRSVP", ATTACH = pst_udps, CONSUME = UDP_sizeof, NEXT = d_rsvp } =;
DECODE d_rsvp_3( RSVP_udp_port1 ) {
	PLEVEL = HL_RSVP,
	TITLE = "Encapsulated RSVP", BRIEF_TITLE = "EncapRSVP", ATTACH = pst_udpd, CONSUME = UDP_sizeof, NEXT = d_rsvp } =;
DECODE d_rsvp_4( RSVP_udp_port2 ) {
	PLEVEL = HL_RSVP,
	TITLE = "Encapsulated RSVP", BRIEF_TITLE = "EncapRSVP", ATTACH = pst_udpd, CONSUME = UDP_sizeof, NEXT = d_rsvp } =;



# PST: pst_rsvp
# Entry from the command line when using the type switch.
#
PST pst_rsvp(0) { TYPE = "RSVP", DEFAULT = d_rsvp } =;


# DECODE: d_rsvp_ip
# Entry from IP for RSVP packets using the RSVP protocol number.
#
DECODE d_rsvp_ip( RSVP_protocol ) {
	TITLE = "RSVP", ATTACH = pst_ip, CONSUME = $32, NEXT = d_rsvp } =;

